@@ -23,7 +23,11 @@ cmCTestBuildCommand::cmCTestBuildCommand()
 {
   this->GlobalGenerator = 0;
   this->Arguments[ctb_NUMBER_ERRORS] = "NUMBER_ERRORS";
-  this->Arguments[ctb_NUMBER_WARNINGS] = "NUMBER_WARNINGS"; 
+  this->Arguments[ctb_NUMBER_WARNINGS] = "NUMBER_WARNINGS";
+  this->Arguments[ctb_TARGET] = "TARGET";
+  this->Arguments[ctb_CONFIGURATION] = "CONFIGURATION";
+  this->Arguments[ctb_FLAGS] = "FLAGS";
+  this->Arguments[ctb_PROJECT_NAME] = "PROJECT_NAME";
   this->Arguments[ctb_LAST] = 0;
   this->Last = ctb_LAST;
 }
@@ -49,6 +53,7 @@ cmCTestGenericHandler* cmCTestBuildCommand::InitializeHandler()
     return 0;
     }
   this->Handler =  (cmCTestBuildHandler*)handler;
+
   const char* ctestBuildCommand
     = this->Makefile->GetDefinition("CTEST_BUILD_COMMAND");
   if ( ctestBuildCommand && *ctestBuildCommand )
@@ -60,13 +65,33 @@ cmCTestGenericHandler* cmCTestBuildCommand::InitializeHandler()
     const char* cmakeGeneratorName
       = this->Makefile->GetDefinition("CTEST_CMAKE_GENERATOR");
     const char* cmakeProjectName
-      = this->Makefile->GetDefinition("CTEST_PROJECT_NAME");
-    const char* cmakeBuildConfiguration
+      = (this->Values[ctb_PROJECT_NAME] && *this->Values[ctb_PROJECT_NAME])
+      ? this->Values[ctb_PROJECT_NAME]
+      : this->Makefile->GetDefinition("CTEST_PROJECT_NAME");
+
+    // Build configuration is determined by: CONFIGURATION argument,
+    // or CTEST_BUILD_CONFIGURATION script variable, or
+    // CTEST_CONFIGURATION_TYPE script variable, or ctest -C command
+    // line argument... in that order.
+    //
+    const char* ctestBuildConfiguration
       = this->Makefile->GetDefinition("CTEST_BUILD_CONFIGURATION");
+    const char* cmakeBuildConfiguration
+      = (this->Values[ctb_CONFIGURATION] && *this->Values[ctb_CONFIGURATION])
+      ? this->Values[ctb_CONFIGURATION]
+      : ((ctestBuildConfiguration && *ctestBuildConfiguration)
+        ? ctestBuildConfiguration
+        : this->CTest->GetConfigType().c_str());
+
     const char* cmakeBuildAdditionalFlags
-      = this->Makefile->GetDefinition("CTEST_BUILD_FLAGS");
+      = (this->Values[ctb_FLAGS] && *this->Values[ctb_FLAGS])
+      ? this->Values[ctb_FLAGS]
+      : this->Makefile->GetDefinition("CTEST_BUILD_FLAGS");
     const char* cmakeBuildTarget
-      = this->Makefile->GetDefinition("CTEST_BUILD_TARGET");
+      = (this->Values[ctb_TARGET] && *this->Values[ctb_TARGET])
+      ? this->Values[ctb_TARGET]
+      : this->Makefile->GetDefinition("CTEST_BUILD_TARGET");
+
     if ( cmakeGeneratorName && *cmakeGeneratorName &&
       cmakeProjectName && *cmakeProjectName )
       {
@@ -104,7 +129,7 @@ cmCTestGenericHandler* cmCTestBuildCommand::InitializeHandler()
           }
         cmakeBuildConfiguration = config;
         }
-      
+
       std::string buildCommand
         = this->GlobalGenerator->
         GenerateBuildCommand(cmakeMakeProgram,
@@ -119,10 +144,17 @@ cmCTestGenericHandler* cmCTestBuildCommand::InitializeHandler()
     else
       {
       cmOStringStream ostr;
-      ostr << "CTEST_BUILD_COMMAND or CTEST_CMAKE_GENERATOR not specified. "
-        "Please specify the CTEST_CMAKE_GENERATOR and CTEST_PROJECT_NAME if "
-        "this is a CMake project, or specify the CTEST_BUILD_COMMAND for "
-        "cmake or any other project.";
+      ostr << "has no project to build. If this is a "
+        "\"built with CMake\" project, verify that CTEST_CMAKE_GENERATOR "
+        "and CTEST_PROJECT_NAME are set."
+        "\n"
+        "CTEST_PROJECT_NAME is usually set in CTestConfig.cmake. Verify "
+        "that CTestConfig.cmake exists, or CTEST_PROJECT_NAME "
+        "is set in the script, or PROJECT_NAME is passed as an argument "
+        "to ctest_build."
+        "\n"
+        "Alternatively, set CTEST_BUILD_COMMAND to build the project "
+        "with a custom command line.";
       this->SetError(ostr.str().c_str());
       return 0;
       }
@@ -143,7 +175,7 @@ bool cmCTestBuildCommand::InitialPass(std::vector<std::string> const& args,
 {
   bool ret =  cmCTestHandlerCommand::InitialPass(args, status);
   if ( this->Values[ctb_NUMBER_ERRORS] && *this->Values[ctb_NUMBER_ERRORS])
-    {  
+    {
     cmOStringStream str;
     str << this->Handler->GetTotalErrors();
     this->Makefile->AddDefinition(
