@@ -9,28 +9,6 @@
   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the License for more information.
 ============================================================================*/
-// This class represents a cmake invocation. It is the top level class when
-// running cmake. Most cmake based GUIS should primarily create an instance
-// of this class and communicate with it.
-//
-// The basic process for a GUI is as follows:
-//
-// 1) Create a cmake instance
-// 2) Set the Home & Start directories, generator, and cmake command. this
-//    can be done using the Set methods or by using SetArgs and passing in
-//    command line arguments.
-// 3) Load the cache by calling LoadCache (duh)
-// 4) if you are using command line arguments with -D or -C flags then
-//    call SetCacheArgs (or if for some other reason you want to modify the
-//    cache, do it now.
-// 5) Finally call Configure
-// 6) Let the user change values and go back to step 5
-// 7) call Generate
-//
-// If your GUI allows the user to change the start & home directories then
-// you must at a minimum redo steps 2 through 7.
-//
-
 
 #ifndef cmake_h
 #define cmake_h
@@ -39,6 +17,7 @@
 #include "cmPropertyDefinitionMap.h"
 #include "cmPropertyMap.h"
 
+class cmGlobalGeneratorFactory;
 class cmGlobalGenerator;
 class cmLocalGenerator;
 class cmCacheManager;
@@ -53,6 +32,30 @@ class cmListFileBacktrace;
 class cmTarget;
 class cmGeneratedFileStream;
 
+/** \brief Represents a cmake invocation.
+ *
+ * This class represents a cmake invocation. It is the top level class when
+ * running cmake. Most cmake based GUIS should primarily create an instance
+ * of this class and communicate with it.
+ *
+ * The basic process for a GUI is as follows:
+ *
+ * -# Create a cmake instance
+ * -# Set the Home & Start directories, generator, and cmake command. this
+ *    can be done using the Set methods or by using SetArgs and passing in
+ *    command line arguments.
+ * -# Load the cache by calling LoadCache (duh)
+ * -# if you are using command line arguments with -D or -C flags then
+ *    call SetCacheArgs (or if for some other reason you want to modify the
+ *    cache), do it now.
+ * -# Finally call Configure
+ * -# Let the user change values and go back to step 5
+ * -# call Generate
+
+ * If your GUI allows the user to change the start & home directories then
+ * you must at a minimum redo steps 2 through 7.
+ */
+
 class cmake
 {
  public:
@@ -66,31 +69,33 @@ class cmake
   };
 
 
-  /** Describes the working modes of cmake.
-   * NORMAL_MODE: cmake runs to create project files
-   * SCRIPT_MODE: in script mode there is no generator and no cache. Also,
-   *              language are not enabled, so add_executable and things do
-   *              not do anything. Started by using -P
-   * FIND_PACKAGE_MODE: cmake runs in pkg-config like mode, i.e. it just
-   *              searches for a package and prints the results to stdout.
-   *              This is similar to SCRIPT_MODE, but commands like
-   *              add_library() work too, since they may be used e.g. in
-   *              exported target files. Started via --find-package
-   */
+  /** \brief Describes the working modes of cmake */
   enum WorkingMode
   {
-    NORMAL_MODE,
+    NORMAL_MODE, ///< Cmake runs to create project files
+    /** \brief Script mode (started by using -P).
+     *
+     * In script mode there is no generator and no cache. Also,
+     * languages are not enabled, so add_executable and things do
+     * nothing.
+     */
     SCRIPT_MODE,
+    /** \brief A pkg-config like mode
+     *
+     * In this mode cmake just searches for a package and prints the results to
+     * stdout. This is similar to SCRIPT_MODE, but commands like add_library()
+     * work too, since they may be used e.g. in exported target files. Started
+     * via --find-package.
+     */
     FIND_PACKAGE_MODE
   };
   typedef std::map<cmStdString, cmCommand*> RegisteredCommandsMap;
 
-  ///! construct an instance of cmake
+  /// Default constructor
   cmake();
-  ///! destruct an instance of cmake
+  /// Destructor
   ~cmake();
 
-  ///! construct an instance of cmake
   static const char *GetCMakeFilesDirectory() {return "/CMakeFiles";};
   static const char *GetCMakeFilesDirectoryPostSlash() {
     return "CMakeFiles/";};
@@ -98,8 +103,8 @@ class cmake
   //@{
   /**
    * Set/Get the home directory (or output directory) in the project. The
-   * home directory is the top directory of the project. It is where
-   * cmake was run. Remember that CMake processes
+   * home directory is the top directory of the project. It is the
+   * path-to-source cmake was run with. Remember that CMake processes
    * CMakeLists files by recursing up the tree starting at the StartDirectory
    * and going up until it reaches the HomeDirectory.
    */
@@ -164,12 +169,6 @@ class cmake
   int Configure();
   int ActualConfigure();
 
-  /**
-   * Configure the cmMakefiles. This routine will create a GlobalGenerator if
-   * one has not already been set. It will then Call Configure on the
-   * GlobalGenerator. This in turn will read in an process all the CMakeList
-   * files for the tree. It will not produce any actual Makefiles, or
-   * workspaces. Generate does that.  */
   int LoadCache();
   void PreLoadCMakeFiles();
 
@@ -188,6 +187,14 @@ class cmake
   ///! Get the names of the current registered generators
   void GetRegisteredGenerators(std::vector<std::string>& names);
 
+  ///! Set the name of the selected generator-specific toolset.
+  void SetGeneratorToolset(std::string const& ts)
+    { this->GeneratorToolset = ts; }
+
+  ///! Get the name of the selected generator-specific toolset.
+  std::string const& GetGeneratorToolset() const
+    { return this->GeneratorToolset; }
+
   ///! get the cmCachemManager used by this invocation of cmake
   cmCacheManager *GetCacheManager() { return this->CacheManager; }
 
@@ -296,7 +303,7 @@ class cmake
   void SetWorkingMode(WorkingMode mode) { this->CurrentWorkingMode = mode; }
   WorkingMode GetWorkingMode() { return this->CurrentWorkingMode; }
 
-  ///! Debug the try compile stuff by not delelting the files
+  ///! Debug the try compile stuff by not deleting the files
   bool GetDebugTryCompile(){return this->DebugTryCompile;}
   void DebugTryCompileOn(){this->DebugTryCompile = true;}
 
@@ -342,6 +349,8 @@ class cmake
                       bool chain = false,
                       const char *variableGroup = 0);
 
+  bool GetIsPropertyDefined(const char *name, cmProperty::ScopeType scope);
+
   // get property definition
   cmPropertyDefinition *GetPropertyDefinition
   (const char *name, cmProperty::ScopeType scope);
@@ -374,7 +383,7 @@ class cmake
   /** Display a message to the user.  */
   void IssueMessage(cmake::MessageType t, std::string const& text,
                     cmListFileBacktrace const& backtrace);
-  //  * run the --build option
+  ///! run the --build option
   int Build(const std::string& dir,
             const std::string& target,
             const std::string& config,
@@ -398,12 +407,9 @@ protected:
      cmExternalMakefileProjectGenerator* (*CreateExtraGeneratorFunctionType)();
   typedef std::map<cmStdString,
                 CreateExtraGeneratorFunctionType> RegisteredExtraGeneratorsMap;
-
-  typedef cmGlobalGenerator* (*CreateGeneratorFunctionType)();
-  typedef std::map<cmStdString,
-                   CreateGeneratorFunctionType> RegisteredGeneratorsMap;
+  typedef std::vector<cmGlobalGeneratorFactory*> RegisteredGeneratorsVector;
   RegisteredCommandsMap Commands;
-  RegisteredGeneratorsMap Generators;
+  RegisteredGeneratorsVector Generators;
   RegisteredExtraGeneratorsMap ExtraGenerators;
   void AddDefaultCommands();
   void AddDefaultGenerators();
@@ -420,6 +426,7 @@ protected:
   std::string StartOutputDirectory;
   bool SuppressDevWarnings;
   bool DoSuppressDevWarnings;
+  std::string GeneratorToolset;
 
   ///! read in a cmake list file to initialize the cache
   void ReadListFile(const std::vector<std::string>& args, const char *path);
@@ -438,7 +445,7 @@ protected:
   void SetDirectoriesFromFile(const char* arg);
 
   //! Make sure all commands are what they say they are and there is no
-  //macros.
+  /// macros.
   void CleanupCommandsAndMacros();
 
   void GenerateGraphViz(const char* fileName) const;
@@ -449,6 +456,8 @@ protected:
                               std::string const& link);
   static int ExecuteEchoColor(std::vector<std::string>& args);
   static int ExecuteLinkScript(std::vector<std::string>& args);
+  static int WindowsCEEnvironment(const char* version,
+                                  const std::string& name);
   static int VisualStudioLink(std::vector<std::string>& args, int type);
   static int VisualStudioLinkIncremental(std::vector<std::string>& args,
                                          int type,
@@ -468,8 +477,6 @@ protected:
 
   ///! Find the full path to one of the cmake programs like ctest, cpack, etc.
   std::string FindCMakeProgram(const char* name) const;
-
-  bool RejectUnsupportedPaths(const char* desc, std::string const& path);
 private:
   cmake(const cmake&);  // Not implemented.
   void operator=(const cmake&);  // Not implemented.
@@ -530,6 +537,13 @@ private:
    "A makefile generator is responsible for generating a particular build " \
    "system.  Possible generator names are specified in the Generators " \
    "section."},\
+  {"-T <toolset-name>", "Specify toolset name if supported by generator.", \
+   "Some CMake generators support a toolset name to be given to the " \
+   "native build system to choose a compiler.  " \
+   "This is supported only on specific generators:\n" \
+   "  Visual Studio >= 10\n" \
+   "  Xcode >= 3.0\n" \
+   "See native build system documentation for allowed toolset names."}, \
   {"-Wno-dev", "Suppress developer warnings.",\
    "Suppress warnings that are meant for the author"\
    " of the CMakeLists.txt files."},\
